// nlMediaInfo.js 101016
(function() {

window.nlMediaInfo = {
	/**
	 * 情報テキストのデフォルトテンプレート
	 */
	TEXT_TMPL: new Template("テンプレートを設定してください"),
	/**
	 * 情報を取得してオブジェクトにキャッシュする
	 */
	get: function(video_id, thread_id) {
		if (typeof this[video_id] == 'undefined') {
			var data = {};
			this.getCacheInfo(data, video_id);
			if (data.cacheNowDL == "0") {
				this.getThumbInfo(data, thread_id || video_id);
				this.getMediaInfo(data, video_id + data.cacheType);
				this.extend(data);
			}
			this[video_id] = data;
		}
		return this[video_id];
	},
	/**
	 * テンプレートに従って情報を更新する(watchページのみ)
	 */
	update: function() {
		if (typeof window.Video == 'object') {
			var tab  = $("itab_nl_mediainfo");
			var text = $("nl_mediainfo_text");
			if(this.executer) this.executer.stop();
			if(this.write(text)) return;
			var self = this;
			this.executer = new PeriodicalExecuter(function() {
				if(tab && tab.getStyle("display") == 'block') {
					if(self.write(text)) self.executer.stop();
				}
			}, 5);
		}
	},
	/**
	 * 情報を取得してテキスト要素に書き込む
	 */
	write: function(text) {
		if (!text || !text.visible())
			return true; // テキスト要素が無効なら終了
		var data = this.get(Video.id, Video.v);
		if (typeof data.cacheNowDL == 'undefined') {
			text.update("<b>キャッシュがありません</b>");
		} else if (data.cacheNowDL != "0") {
			var n = Number(data.cacheSize) * 100 / Number(data.cacheNowDL);
			text.update("<b>キャッシュ中です... " + Math.round(n) + "%</b>");
		} else {
			text.update(this.TEXT_TMPL.evaluate(data));
			return true;
		}
		delete this[Video.id]; // 再取得のためキャッシュした情報を削除
		return false;
	},
	/**
	 * APIを同期呼び出しする
	 */
	callAPI: function(path, callback) {
		new Ajax.Request(path, {
			method: "GET",
			asynchronous: false,
			onSuccess: callback
		});
	},
	/**
	 * キャッシュ情報を取得してdataに設定する
	 */
	getCacheInfo: function(data, id) {
		this.callAPI("/cache/ajax_info?" + id, function(r) {
			data.cacheinfoText = r.responseText;
			if (r.responseText.indexOf("OK") == 0) {
				var s = r.responseText.split(",", 6);
				data.cacheType  = s[1];
				data.cacheExt   = s[2];
				data.cacheSize  = s[3];
				data.cacheNowDL = s[4];
				data.cacheTitle = s[5];
			}
		});
	},
	/**
	 * thumbinfoを取得してdataに設定する
	 */
	getThumbInfo: function(data, id) {
		var self = this;
		this.callAPI("/api/getthumbinfo/" + id, function(r) {
			data.thumbinfoText = r.responseText;
			data.thumbinfoXML  = r.responseXML;
			self.pickup(data, r.responseXML, 'thumb', "TI_");
		});
	},
	/**
	 * MediaInfoを取得してdataに設定する
	 */
	getMediaInfo: function(data, id) {
		var self = this;
		this.callAPI("/cache/mediainfo?" + id, function(r) {
			data.mediainfoText = r.responseText;
			data.mediainfoXML  = r.responseXML;
			self.pickup(data, r.responseXML, 'track[type="General"]', 'MI_General_');
			self.pickup(data, r.responseXML, 'track[type="Video"]'  , 'MI_Video_'  );
			self.pickup(data, r.responseXML, 'track[type="Audio"]'  , 'MI_Audio_'  );
		});
	},
	/**
	 * XML文書からCSSセレクタで抽出した最初の要素をdataに設定
	 */
	pickup: function(data, element, selector, prefix) {
		var node;
		if (element.querySelector) {
			node = element.querySelector(selector);
		} else {
			// IEはelement.querySelectorをサポートしていないので超適当に抽出
			var name = selector, attr, value;
			if (selector.match(/(\w+)\[(\w+)="(\w+)"\]/)) {
				name = RegExp.$1, attr = RegExp.$2, value = RegExp.$3;
			}
			var nodes = element.getElementsByTagName(name);
			if (attr && value) {
				for (i = 0; i < nodes.length; i++) {
					if(!nodes[i].attributes) continue;
					var c = nodes[i].attributes[0];
					if (c.name == attr && c.value == value) {
						node = nodes[i]; break;
					}
				}
			} else {
				node = nodes[0];
			}
		}
		if(!node) return;
		for (i = 0; i < node.childNodes.length; i++) {
			var c = node.childNodes[i];
			if (c.childNodes && c.childNodes.length == 1) {
				var name = c.nodeName, value = c.textContent||c.text;
				if(!name || !value) continue;
				var key = (prefix||"") + name;
				if (key.indexOf("MI_") == 0)
					value = value.replace(/(\d+) (\d)/, "$1,$2");
				data[key] = value;
			}
		}
	},
	/**
	 * 追加のテキスト要素をdataに設定
	 * 
	 * size_high     : 標準画質のサイズ(MB)
	 * size_low      : エコノミー画質のサイズ(MB)
	 * video_bitrate : 映像ビットレート(モード付き)
	 * audio_bitrate : 音声ビットレート(モード付き)
	 * resolution    : 映像の解像度(WxH)
	 */
	extend: function(data) {
		data.size_high = Number(data.TI_size_high) > 0 ?
			this.str_m(data.TI_size_high) + "iB" : "None";
		data.size_low  = Number(data.TI_size_low ) > 0 ?
			this.str_m(data.TI_size_low ) + "iB" : "None";
		if (data.TI_length && data.TI_length.match(/(\d+):(\d+)/)) {
			var length = Number(RegExp.$1) * 60 + Number(RegExp.$2);
			if (length > 0 && data.TI_size_high) {
				data.bitrate = this.str_k(Number(data.TI_size_high) * 8 / length) + "bps";
			}
		}
		if (data.MI_Video_Bit_rate) {
			var s = data.MI_Video_Bit_rate;
			if (data.MI_Video_Bit_rate_mode)
				s += "(" + data.MI_Video_Bit_rate_mode + ")";
			data.video_bitrate = s;
		}
		if (data.MI_Audio_Bit_rate) {
			var s = data.MI_Audio_Bit_rate;
			if (data.MI_Audio_Bit_rate_mode)
				s += "(" + data.MI_Audio_Bit_rate_mode + ")";
			data.audio_bitrate = s;
		}
		if (data.MI_Video_Width && data.MI_Video_Height) {
			var width  = data.MI_Video_Width.replace(/[a-z\s]+/g, "");
			var height = data.MI_Video_Height.replace(/[a-z\s]+/g, "");
			data.resolution = width + "x" + height;
		}
	},
	str_k: function(n) {
		var s = String(Number(n) / 1024);
		return s.substring(0, s.indexOf(".") + 2) + " K";
	},
	str_m: function(n) {
		var s = String(n / (1024 * 1024));
		return s.substring(0, s.indexOf(".") + 2) + " M";
	},
	empty: function() {}
};

})();